/* btrc - Bluetooth Remote Control
 * Copyright (C) 2004 Rossen Tchobanski (rosko.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <signal.h>

#include <stdio.h>

#include <libxml/parser.h>
#include <libxml/tree.h>
#include <ezV24/ezV24.h>

#define RFCOMM      "/dev/rfcomm0"
#define XML_FILE    "./btrc.xml"
                                                                                         
#define NAME_LEN    50
#define CMD_LEN     255
#define BUF_LEN     500

#define NO_MENU	    65536

#define TRUE        (1==1)
#define FALSE       (!TRUE)


xmlDoc *doc = NULL;	        
v24_port_t *port = NULL;

typedef char bool;
  
typedef struct node_s node_t;
struct node_s
{
  char name[NAME_LEN];
  char action[CMD_LEN];
  xmlNode *node;
  char *menu;
};	  

/*
** Procedure: Signalhandler
**
** Description:
**
*/
static void Signalhandler ( int reason )
{
  v24ClosePort(port);
  xmlFreeDoc(doc);     
  xmlCleanupParser();  

  exit(99);
}
                                                                                               
/*
** Procedure: installSignalhandler
**
** Description:
**
*/
static void installSignalhandler ( void )
{
  signal(SIGINT,Signalhandler);   
  signal(SIGTERM,Signalhandler);  
}

/*
** Procedure: print_element_name
**
** Description:
**
*/
static void
print_element_name(xmlNode * a_node)
{
  xmlNode *cur_node = a_node;

  xmlChar *name = xmlGetProp( cur_node, BAD_CAST "name" );      	
  xmlChar *action = xmlGetProp( cur_node, BAD_CAST "action" );  	
  printf("%s, '%s' '%s'\n", cur_node->name, name, action);			        	
}

/*
** Procedure: print_element_names
**
** Description:
**
*/
static void
print_element_names(xmlNode * a_node)
{
  xmlNode *cur_node = a_node;					  

  for (cur_node = a_node; cur_node; cur_node = cur_node->next) {  
    if (cur_node->type == XML_ELEMENT_NODE) {			  
    	print_element_name(cur_node);
    }								  

    print_element_names(cur_node->children);			  
  }								  
}

/*
** Procedure: get_element
**
** Description:
**
*/
static node_t *
get_element(xmlNode *a_node)
{
  static node_t node;
  xmlNode *cur_node = a_node; 
  
  node.name[0]=0;
  node.action[0]=0;
  node.node=NULL;
  
  for (; cur_node; cur_node = cur_node->next) {  
    if (cur_node->type == XML_ELEMENT_NODE) {				     
      xmlChar *name = xmlGetProp( cur_node, BAD_CAST "name" );       
      xmlChar *action = xmlGetProp( cur_node, BAD_CAST "action" );  
      
      sprintf(node.name, "%s", name);
      sprintf(node.action, "%s", action);
      node.node = cur_node;

      return &node;	
    }									     
  }

  return 0;					        
}

/*
** Procedure: get_menu
**
** Description:
**
*/
static node_t *
get_menu(char *buf, xmlNode *a_node, node_t l[])
{
  int nr = 0;
  char *l_buf = malloc(BUF_LEN);
  node_t *cur_node;
  
  buf[0] = 0;
  l_buf[0] = 0;
  
  /*
    l[0] = title
    l[1] = first menu item
    ....
  */

  
  cur_node = get_element(a_node); 
  sprintf(l[nr].name, "%s", cur_node->name);    	  
  sprintf(l[nr].action, "%s", cur_node->action);	  
  nr++; 

  cur_node->node = a_node->children;
  
  for (; cur_node->node; cur_node->node = cur_node->node->next) {  
    if (cur_node->node->type == XML_ELEMENT_NODE) {				     
      cur_node = get_element(cur_node->node);
      
      if (cur_node) {
        sprintf(l[nr].name, "%s", cur_node->name);  	
        sprintf(l[nr].action, "%s", cur_node->action);
	l[nr].node = cur_node->node;
      } 	
        						
      sprintf(l_buf, "%s,\"%s\"", l_buf, cur_node->name);	
      nr++; 
    }					      
  }
  
  l[nr].name[0] = 0;	      
  l[nr].action[0] = 0;    

  sprintf(buf, "AT*EASM=\"%s\",0,1,%d%s\r", l[0].name, nr-1, l_buf);
  
  cur_node->menu = buf;
  free(l_buf);
 
  return cur_node;
}

/*
** Procedure: make_menu
**
** Description:
**
*/
static node_t *
make_menu(char *buf, node_t l[], int menu_nr)
{
  char *l_buf = malloc(BUF_LEN);
  int nr; 
  node_t *cur_node;

  buf[0] = 0;
  l_buf[0] = 0;

  for (nr=1; l[nr].name[0] != 0; nr++) {
    sprintf(l_buf, "%s,\"%s\"", l_buf, l[nr].name);
    if (nr == menu_nr) {
      cur_node->node = l[nr].node;
    }
  }
  
  sprintf(buf, "AT*EASM=\"%s\",0,%d,%d%s\r", l[0].name, menu_nr, nr-1, l_buf);
  cur_node->menu = buf;
  
  return cur_node;
}

/*
** Procedure: make_menu
**
** Description:
**
*/
int
run_action(xmlNode * a_node)
{
  xmlNode *cur_node = NULL; 
  
  cur_node = a_node; 
  print_element_name(cur_node);
  xmlChar *name = xmlGetProp( cur_node, BAD_CAST "name" );	     
  xmlChar *action = xmlGetProp( cur_node, BAD_CAST "action" );       
  if (action) {
    system(action);
  }
  
  return 0;
}

/*
** Procedure: get_element_menu
**
** Description:
**
*/
xmlNode *
get_element_menu(xmlNode *a_node)
{
  char buffer[BUF_LEN];	         
  int menu_nr = NO_MENU;
  node_t l[50];
  char cmd[CMD_LEN];
  xmlNode *cur_node = NULL;  
  node_t *sel_node;

  cur_node = a_node;
  buffer[0] = 0;							  

  if (!cur_node) {
    printf("exit 0\n");		  
    return NULL;
  }  

  sprintf(cmd, "%s", get_menu(buffer, cur_node, l)->menu);   
  v24Puts (port, cmd);					  

  while (TRUE) {					  
    v24Gets (port, buffer, BUF_LEN);	
    
    if (!strncmp(buffer,"*EAAI",5)) { 
      /* we are at the root */
      get_element_menu(cur_node);
    }	
  							  
    sscanf(buffer, "*EAMI: %d", &menu_nr);		  
  							  
    if ((menu_nr>0) && (menu_nr<NO_MENU)) {
      sel_node = make_menu(buffer, l, menu_nr);		  
      v24Puts (port, sel_node->menu);	  
      v24Gets (port, buffer, BUF_LEN);			  

      run_action(sel_node->node);
      if (!strcmp(sel_node->node->name, "menu"))
        get_element_menu(sel_node->node);
    } 							  
    if (menu_nr==0) {					  
      run_action(cur_node);
      if (!strcmp(cur_node->name, "btrc")) {
        /* we are at the root */
        continue;
      }			  
      /* return to a menu back */
      get_element_menu(cur_node->parent);		  
    }							  
    menu_nr = NO_MENU;					  
    buffer[0] = 0;							  
  }
}


/***********************************************
** 
**  M A I N
** 
************************************************/
int
main(int argc, char **argv)
{
  xmlNode *root_element = NULL;  
  xmlNode *cur_element = NULL;  

  int rc; 
  char cmd[CMD_LEN];
  char buffer[BUF_LEN];	         
  


  installSignalhandler();    

  /* Init XML *****************************************************/    
  LIBXML_TEST_VERSION					     
  							    
  /*parse the file and get the DOM */
  doc = xmlReadFile(XML_FILE, NULL, 0);		    
  							    
  if (doc == NULL) {
      printf("error: could not parse file %s\n", XML_FILE);  
  }							    
  							    
  root_element = xmlDocGetRootElement(doc);

  
  /* Init v24 ******************************************************/
  port = v24OpenPort (RFCOMM, V24_STANDARD);
  if ( port==NULL )
  {
      printf("error: sorry, OpenPort failed!\n");
      return 0;
  }

  rc=v24SetParameters(port,V24_B4800,V24_8BIT,V24_NONE);
  if ( rc!=V24_E_OK )
  {
      printf("error: sorry, SetParameters failed!\n");
      return 0;
  }

  
  /* Start ROOT menu ************************************************/
  sprintf(cmd, "AT*EAM=\"%s\"\r", get_element(root_element)->name); 
  v24Puts (port, cmd);

  cur_element = root_element;
  
  printf("%s started.\n", argv[0]);

  get_element_menu(root_element);
  
    

  xmlFreeDoc(doc);     
  xmlCleanupParser();  

  v24ClosePort (port);

  return 0;	       
}

